
version 17 

**************************************************************************
** Step 1.1:  Preparing University-Admitted dataset only***
**************************************************************************

*Import the dataset 
import excel "${data}/FD_Uni_eng.xlsx", sheet("uni") firstrow clear

**** Drop applicants duplicates
drop if test==.
sort kankorID, stable
quietly by kankorID:  gen dup = cond(_N==1,0,_n)
ta dup
drop if dup>1

merge 1:1 kankorID using "${data}/uni_admitted_DIST.dta", gen(merge_dist_id) // merge district ID participants KankorID

rename dist_ID nsia_dist
merge m:1 nsia_dist using "${data}/dist_light.dta", gen(merg_nitLit) // nightlight average 2014-2015, using NSIA_dist code

encode highschool, gen(hs)
encode concen , gen(concent)
encode major, gen(major1)
encode univer, gen(uni)
encode pro1, gen(province)
encode gender, gen (gend)
encode ihe, gen(ihetype)
encode uniloc, gen(unilocation)

order gradyear1 test concent major1 portiongp uni unilocation gp remove province gend year_kankor hs ihetype

label var kankorID "Kankor exam ID"
label var gradyear1 "gap year between HS and year of kankor exam"
label var test "Kankor score 0-360"
label var concent "concentration - admitted unit of study"
label var major1 "major as the baseyear 2013"
label var portiongp "share of concentrations treated within a major"
label var uni "University"
label var remove "1 if opted-out of the policy, 0 otherwise"
label var province "province - where an applicant participated in the exam"
label var gend "gender, 1 female 2 Male" 
labe var year_kankor "Marticulation year"
lab var ihetype "Type of IHE: uni=4y public IHE, asso=2y institutions, noresult=failed"
lab var hs "highschool name" 
lab var unilocation "province name niversity is located"
label var gp "time-variant treatment inidcator; 1 adopted quota in time t, 0 otherwise"

***** CORRECT TREATMENT indicator EROR 
replace gp=0 if concent==502 | concent==735 | concent==328 | concent==863


***** correct educational and Islamic Studies that were Gender-specific without the policy
replace gp=0 if concent==460 | concent==550 | concent==566 | concent==568 | concent==571 | concent==575 | concent==682 | concent==904 | concent==908 | concent==917 | concent==567 | concent==744 | concent==560

replace portiongp=0 if concent==460 | concent==550 | concent==566 | concent==568 | concent==571 | concent==575 | concent==682 | concent==904 | concent==908 | concent==917 | concent==567 | concent==744 | concent==560


drop highschool concen major univer uniloc pro1 gender ihe // drop temp vars
rename major1 major
rename gend gender

***** Generate academic programs identifier
gen fields="Agriculture" if major>=2 & major<=27 | major==301

replace fields="Computer Science" if major>=49 & major<=60 | major==174

replace fields="Economics" if major>=66 & major<=85

replace fields="Education" if major==36 | major==37 | major==42 | major==42 | major>=86 & major<=114 | major==143 | major==144 | major==162  | major==185  | major==215 | major==216  | major==217  | major==250 | major==35 | major==40 | major==41 | major==43 | major==133 | major==214 | major==249 | major==253 | major==254 | major==280

replace fields="Engineering" if major==22 | major==33 | major==46 | major==47 | major==49 | major==60 | major>=115 & major<=132 | major==155 | major==218 | major==231 | major==288 | major==293 | major==302 | major==38 | major==48 | major==157 | major==233 | major==258 | major==292

replace fields="Geology and Environment" if major==135 | major>=145 & major<=153 | major==155  | major==158  | major==159  | major==228  | major==234  | major==291 | major==134

replace fields="Islamic Studies" if major>=163 & major<=173  | major==270 | major==271 | major==175 | major==266 | major==267 | major==268 | major==269 | major==30

replace fields="Journalism" if major>=176 & major<=184

replace fields="Law and Political Science" if major>=186 & major<=199

replace fields="Literature" if major==30 | major==31 | major==63 | major==64 | major==131 | major==132 | major==137 | major==140  | major==154 | major==162  | major>=200 & major<=213 | major==242 | major==243 | major==259 | major==260 | major==282 | major==289 | major==290 | major==44 

replace fields="Medical Studies" if major>=219 & major <=227 | major>=283 & major<=286 | major==28 | major==257 | major==232 | major==245 | major==246

replace fields="Psychology" if major==1 | major==37 | major==62 | major==65 | major==142 | major==244  | major==251 | major==255 | major==256  | major==279 | major==61

replace fields="Public Affairs" if major>=236 & major<=240

replace fields="Science" if major==141 | major==230  | major==261 | major==262 

replace fields="Social Science" if major==29 | major==138 | major==139 | major==147 | major==247 | major==263 | major==264 | major==265 | major>=272 & major<=278 | major==34 | major==45 | major==156 | major==229 | major==241 | major==248 | major==252 | major==281 | major==287

replace fields="Technician" if major==136 | major==160 | major==161  | major==235 | major==39

replace fields="Veterinary" if major>=294 & major<=301

**** encode field to acdem_program
encode fields, generate(acdem_program)
lab var acdem_program "Academic program identifier"

**** Competitive programs - CEELM 
gen ceelm=(acdem_program==2 | acdem_program==3 | acdem_program==5 | acdem_program==9 | acdem_program==11)
lab var ceelm "1 if a computer_sci, econ, engineering, law_political science, and Medical studies, 0 otherwise"

gen stem=(acdem_program==1 | acdem_program==2 | acdem_program==5 | acdem_program==11 | acdem_program==12 | acdem_program==14 | acdem_program==16 | acdem_program==17) 
lab var stem "1 STEM academic programs, 0 otherwise"

* time invariante major & concentration level indicators 
bysort concent: egen everconcent=max(gp)
lab var everconcent "ever treated concentrations"

bysort major: egen evermajor=max(gp)
lab var evermajor "ever treated majors"

save "${data}/uni.dta", replace // save Uni admitted applicants list


*******************************************************************************
** Step 1.2:  Preparing Non-University dataset only***
*******************************************************************************

*** Import Non-University dataset 
import delimited "${data}/FD_Non_uni_eng.csv", varnames(1) case(preserve) encoding(UTF-8) clear

encode highschool, generate(hs)
encode univer, gen(uni)
encode pro1, generate(province)
encode gender, gen (gend)
encode ihe, gen(ihetype)

rename concen concent
rename uniloc unilocation

order gradyear1 test concent major portiongp uni unilocation gp remove province gend year_kankor hs ihetype

label var kankorID "Kankor exam ID"
label var gradyear1 "gap year between HS and year of kankor exam"
label var test "Kankor exam 0-360"
label var concent "concentration - admitted unit of study"
label var major "major as the baseyear 2013"
label var portiongp "share of concentrations treated within a major"
label var uni "university, check notes for some additional"
note: uni in combined/nonuni dataset [REG=religious 2y institutions, TVET=Technical and Vocational Edu & Training, TEACH=2y teaching institutions, INST=In 2017 (refers to REG, TVET, TEACH), NoResult=TestTakers who faild the exam]
label var gp "time-variant treatment inidcator; 1 adopted quota in time t, 0 otherwise"
label var remove "1 if opted-out of the policy, 0 otherwise"
label var province "province - where an applicant participated in the exam"
label var gend "1 female 2 Male" 
labe var year_kankor "marticulation year"
lab var ihetype "Type of IHE: uni=4y public IHE, asso=2y institutions, noresult=failed"
lab var hs "highschool name" 
lab var unilocation "province name niversity is located"

drop highschool univer pro1 gender ihe // drop temp vars

rename gend gender

order gradyear1 test concent major portiongp uni unilocation gp remove province gender year_kankor hs ihetype

save "${data}/nonuni.dta", replace

*****************************************************************************
** Step 1.3: Combine university & non-university dataset 
*****************************************************************************

use "${data}/uni.dta", clear
replace ihetype=3 if ihetype==1 // to properly append it with non-uni dataset
label define ihetype_1 1 "asso" 2 "noresult" 3 "uni"
label values ihetype ihetype_1

append using "${data}/nonuni.dta",
label values ihetype ihetype_1

save "${data}/kankor_all.dta", replace 




*****************************************************************************
** Step 1.4: creating concentration-level dataset
*****************************************************************************

use "${data}/uni.dta", clear

* -------------- *
*** Outcome # 1: Create Share of Female and Male APPLICANTS
* -------------- *
sort kankorID, stable

gen one=1
bysort year_kankor concent gender: egen num_gen_concent=sum(one) // number of male or female student
bysort year_kankor concent: egen num_gen_concent_all=sum(one) // total number of students
lab var num_gen_concent_all "total students"
gen gen_percent_concent=(num_gen_concent/num_gen_concent_all) // share of male/female

* Female APPLICANTS
bysort year_kankor concent: gen fs=gen_percent_concent if gender==1
bysort year_kankor concent: egen share_female=max(fs)
lab var share_female "female share by concentration"
replace share_female=0 if share_female==.

*** number of female APPLICANTS
bysort year_kankor concent: egen num_female=max(num_gen_concent) if gender==1
bysort year_kankor concent: egen female_num=max(num_female)
lab var female_num "actual number of female by concentration"

* Male APPLICANTS
bysort year_kankor concent: gen ms=gen_percent_concent if gender==2
bysort year_kankor concent: egen share_male=max(ms)
lab var share_male "male share by concentration"
replace share_male=0 if share_male==.

*** number male APPLICANTS
bysort year_kankor concent: egen num_male=max(num_gen_concent) if gender==2
bysort year_kankor concent: egen male_num=max(num_male)
lab var male_num "actual number of male by concentration"


* -------------- *
*** Outcome #2: Share female and Male APPLICANTS low-socioeconomic background
* -------------- *
bysort year_kankor concent gender dis_low_avg: egen num_f_con=sum(one) if nsia_dist!=. // number of male or female student by district status 

*************************
*** Share Female APPLICANTS from LSES
bysort year_kankor concent: gen su_fem_con=num_f_con if gender==1 & dis_low_avg==1 & nsia_dist!=. // # female applicants from low-socioeconomic districts 

bysort year_kankor concent: egen fem_LSE_con=max(su_fem_con) // # of female from LSES in concentration

gen per_loweco=(fem_LSE_con/num_gen_concent_all) // share LSES FEMALE applicatns
bysort year_kankor concent: egen share_female_lowSE=max(per_loweco) // share female from low-SE
lab var share_female_lowSE "share of female from low-socioeconomic districts by concentration"
replace share_female_lowSE=0 if share_female_lowSE==. // replace missing to zero, since there were no observation for that year 

*************************
*** Share Male APPLICANTS from LSES
bysort year_kankor concent: gen su_ma_con=num_f_con if gender==2 & dis_low_avg==1 & nsia_dist!=. // # MALE applicants from low-socioeconomic districts 

bysort year_kankor concent: egen mal_LSE_con=max(su_ma_con) // # of male from LSES in concentration

gen per_loweco_mal=(mal_LSE_con/num_gen_concent_all) // share LSES male applicatns
bysort year_kankor concent: egen share_male_lowSE=max(per_loweco_mal) // share male from low-SE
lab var share_male_lowSE "share of male applicants from low-socioeconomic districts by concentration"
replace share_male_lowSE=0 if share_male_lowSE==. // replace missing to zero, since there were no observation for that year 

*** number of female LSES
bysort year_kankor concent: egen num_female_lses=max(num_gen_concent) if gender==1 & dis_low_avg==1
bysort year_kankor concent: egen female_num_lses=max(num_female_lses)
lab var female_num_lses "actual number of female by concentration from LSES"
replace female_num_lses=0 if female_num_lses==.

* -------------- *
*** Outcome #3: Overall average score 
* -------------- *
/*
rename test score

bysort concent year_kankor: egen con_avg_scr = mean(score)
bysort concent year_kankor: egen con_sd_scr = sd(score)
bysort concent year_kankor: gen test = (score - con_avg_scr) / con_sd_scr
*/

bysort year_kankor concent: egen avg_score=mean(test) // Overal average score
lab var avg_score "overall average score"

* -------------- *
** Outcome #4: Female and Male Average Score 
* -------------- *
* Female APPLICANTS
bysort year_kankor concent: egen f_score=mean(test) if gender==1
bysort year_kankor concent: egen female_avg_score=max(f_score)
lab var female_avg_score "Female average score by concentration"

** Female Minimum score
bysort year_kankor concent: egen f_score_min=min(test) if gender==1
bysort year_kankor concent: egen female_min_score=max(f_score_min)
lab var female_min_score "Female minimum score by concentration"

** Female Maximum score
bysort year_kankor concent: egen f_score_max=max(test) if gender==1
bysort year_kankor concent: egen female_max_score=max(f_score_max)
lab var female_max_score "Female maximum score by concentration"

* Average Female HSES
bysort year_kankor concent: egen f_score_hses=mean(test) if gender==1 & dis_low_avg==0
bysort year_kankor concent: egen fem_avg_score_HSES=max(f_score_hses)
lab var fem_avg_score_HSES "Female average score by concentration for female coming from high-SES"

* Average Female LSES
bysort year_kankor concent: egen f_sc_lses=mean(test) if gender==1 & dis_low_avg==1
bysort year_kankor concent: egen fem_avg_score_LSES=max(f_sc_lses)
lab var fem_avg_score_LSES "Female average score by concentration for female coming from low-SES"

* Male APPLICANTS average SCORE 
bysort year_kankor concent: egen m_score=mean(test) if gender==2
bysort year_kankor concent: egen male_avg_score=max(m_score) 
lab var male_avg_score "Male average score by concentration"


** Male Minimum score
bysort year_kankor concent: egen m_score_min=min(test) if gender==2
bysort year_kankor concent: egen male_min_score=max(m_score_min)
lab var male_min_score "Male minimum score by concentration"

** Male Maximum score
bysort year_kankor concent: egen m_score_max=max(test) if gender==2
bysort year_kankor concent: egen male_max_score=max(m_score_max)
lab var male_max_score "Male maximum score by concentration"

* Average male HSES
bysort year_kankor concent: egen m_score_hses=mean(test) if gender==2 & dis_low_avg==0
bysort year_kankor concent: egen ma_avg_score_HSES=max(m_score_hses)
lab var ma_avg_score_HSES "Male average score by concentration for male coming from high-SES"

* Average male LSES
bysort year_kankor concent: egen m_sc_lses=mean(test) if gender==2 & dis_low_avg==1
bysort year_kankor concent: egen ma_avg_score_LSES=max(m_sc_lses)
lab var ma_avg_score_LSES "Male average score by concentration for male coming from low-SES"

* -------------- *
** Outcome #5: Female to Male APPLICANTS average score ratio 
* -------------- *

gen fm_ratio= female_avg_score/male_avg_score
lab var fm_ratio "female to male average score ratio"

keep if gender==1

duplicates drop year_kankor concent, force

keep concent gp major ceelm uni unilocation year_kankor fields acdem_program stem dis_low_p50 dis_low_avg pro_cap share_female share_male share_female_lowSE share_male_lowSE avg_score female_avg_score fem_avg_score_HSES fem_avg_score_LSES male_avg_score ma_avg_score_HSES ma_avg_score_LSES fm_ratio female_num male_num female_num_lses num_gen_concent_all female_min_score female_max_score male_min_score male_max_score

order concent gp major ceelm uni unilocation year_kankor fields acdem_program stem dis_low_p50 dis_low_avg pro_cap share_female share_male share_female_lowSE share_male_lowSE avg_score female_avg_score fem_avg_score_HSES fem_avg_score_LSES male_avg_score ma_avg_score_HSES ma_avg_score_LSES fm_ratio female_num male_num female_num_lses num_gen_concent_all female_min_score female_max_score male_min_score male_max_score

* -------------- ** -------------- ** -------------- *
* Create time invariante everconcent indicator
* -------------- ** -------------- ** -------------- *
bysort concent: egen everconcent=max(gp)
lab var everconcent "if concetration ever treated"

* -------------- ** -------------- ** -------------- *
* GENERATE time to EVENT 
* -------------- ** -------------- ** -------------- *
bysort concent: egen con_TY=min(year_kankor) if gp==1
bysort concent: egen event=max(con_TY) 
gen relative= year_kankor - event
lab var relative "relative time to the policy, concent"

order concent gp everconcent relative event 

keep if fm_ratio!=.

save "${data}/uni_by_concentration.dta", replace



*****************************************************************************
** Step 1.5: creating major-level dataset
*****************************************************************************
use "${data}/uni.dta", clear

* -------------- *
*** Outcome # 1: Create Share of Female and Male APPLICANTS
* -------------- *
sort kankorID, stable

gen one=1
bysort year_kankor major gender: egen num_gen_major=sum(one) // number of male or female student
bysort year_kankor major: egen num_gen_all=sum(one) // total number of students
lab var num_gen_all "total students"
gen gen_percent_major=(num_gen_major/num_gen_all) // share of male/female

* Female
bysort year_kankor major: gen fs=gen_percent_major if gender==1
bysort year_kankor major: egen share_female=max(fs)
lab var share_female "female share by major"
replace share_female=0 if share_female==.

* Male 
bysort year_kankor major: gen ms=gen_percent_major if gender==2
bysort year_kankor major: egen share_male=max(ms)
lab var share_male "male share by major"
replace share_male=0 if share_male==.

* -------------- *
*** Outcome #2: Share of female APPLICANTS from low-socioeconomic background
* -------------- *

bysort year_kankor major gender dis_low_avg: egen num_f_con=sum(one) if nsia_dist!=. // number of male or female student by district status 

*************************
*** Share Female APPLICANTS from LSES
bysort year_kankor major: gen su_fem_con=num_f_con if gender==1 & dis_low_avg==1 & nsia_dist!=. // # female from low-socioeconomic districts 

bysort year_kankor concent: egen fem_LSE_maj=max(su_fem_con) // # of female from LSES in major

gen per_loweco=(fem_LSE_maj/num_gen_all)
bysort year_kankor major: egen share_female_lowSE=max(per_loweco) // share female from low-SE
lab var share_female_lowSE "share of female from low-socioeconomic districts by major"
replace share_female_lowSE=0 if share_female_lowSE==. // replace missing to zero, since there were no observation for that year 


*************************
*** Share Male APPLICANTS from LSES
bysort year_kankor major: gen su_ma_maj=num_f_con if gender==2 & dis_low_avg==1 & nsia_dist!=. // # MALE applicants from low-socioeconomic districts 

bysort year_kankor major: egen mal_LSE_maj=max(su_ma_maj) // # of male from LSES in major

gen per_loweco_mal=(mal_LSE_maj/num_gen_all) // share LSES male applicatns
bysort year_kankor major: egen share_male_lowSE=max(per_loweco_mal) // share male from low-SE
lab var share_male_lowSE "share of male applicants from low-socioeconomic districts by concentration"
replace share_male_lowSE=0 if share_male_lowSE==. // replace missing to zero, since there were no observation for that year 


* -------------- *
*** Outcome #3: Overall average score 
* -------------- *
/*
rename test score
bysort major year_kankor: egen mj_avg_scr = mean(score)
bysort major year_kankor: egen mj_sd_scr = sd(score)
bysort major year_kankor: gen test = (score - mj_avg_scr) / mj_sd_scr
*/

bysort year_kankor major: egen avg_score=mean(test) // Overal average score
lab var avg_score "overall average score"

* -------------- *
** Outcome #4: Female and Male APPLICANTS Average Score 
* -------------- *

* Female APPLICANTS
bysort year_kankor major: egen f_score=mean(test) if gender==1
bysort year_kankor major: egen female_avg_score=max(f_score)
lab var female_avg_score "Female average score by major"


* Average Female HSES
bysort year_kankor major: egen f_score_hses=mean(test) if gender==1 & dis_low_avg==0
bysort year_kankor major: egen fem_avg_score_HSES=max(f_score_hses)
lab var fem_avg_score_HSES "Female average score by major for female coming from high-SES"


* Average Female LSES
bysort year_kankor major: egen f_sc_lses=mean(test) if gender==1 & dis_low_avg==1
bysort year_kankor major: egen fem_avg_score_LSES=max(f_sc_lses)
lab var fem_avg_score_LSES "Female average score by major for female coming from low-SES"


* Male APPLICANTS
bysort year_kankor major: egen m_score=mean(test) if gender==2
bysort year_kankor major: egen male_avg_score=max(m_score) 
lab var male_avg_score "Male average score by major"

* Average male HSES
bysort year_kankor major: egen m_score_hses=mean(test) if gender==2 & dis_low_avg==0
bysort year_kankor major: egen ma_avg_score_HSES=max(m_score_hses)
lab var ma_avg_score_HSES "Male average score by major for male coming from high-SES"

* Average male LSES
bysort year_kankor major: egen m_sc_lses=mean(test) if gender==2 & dis_low_avg==1
bysort year_kankor major: egen ma_avg_score_LSES=max(m_sc_lses)
lab var ma_avg_score_LSES "Male average score by major for male coming from low-SES"

* -------------- *
** Outcome #5: Female to Male APPLICANTS average score ratio 
* -------------- *

gen fm_ratio= female_avg_score/male_avg_score
lab var fm_ratio "female to male average score ratio"

bysort year_kankor major: egen treat=max(gp)

bysort year_kankor major: egen por_trat_major=max(portiongp)

drop if treat==.


keep if gender==1

duplicates drop year_kankor major, force

keep major portiongp treat ceelm uni unilocation year_kankor acdem_program stem dis_low_p50 dis_low_avg pro_cap share_female share_male share_female_lowSE share_male_lowSE avg_score female_avg_score fem_avg_score_HSES fem_avg_score_LSES male_avg_score ma_avg_score_HSES ma_avg_score_LSES fm_ratio num_gen_all por_trat_major

order major portiongp treat ceelm uni unilocation year_kankor acdem_program stem dis_low_p50 dis_low_avg pro_cap share_female share_male share_female_lowSE share_male_lowSE avg_score female_avg_score fem_avg_score_HSES fem_avg_score_LSES male_avg_score ma_avg_score_HSES ma_avg_score_LSES fm_ratio num_gen_all por_trat_major

* -------------- ** -------------- ** -------------- *
* Create time invariante evermajor indicator
* -------------- ** -------------- ** -------------- *
replace treat=0 if por_trat_major<=0.33
bysort major: egen evermajor=max(treat)
lab var treat "1 if major adopted the policy, 0 otherwise"
lab var evermajor "if major ever treated"


* -------------- ** -------------- ** -------------- *
* GENERATE time to EVENT 
* -------------- ** -------------- ** -------------- *
bysort major: egen maj_TY=min(year_kankor) if treat==1
bysort major: egen event_maj=max(maj_TY) 
gen relative_maj= year_kankor - event_maj
lab var relative_maj "relative time to the POLICY, major"

order major portiongp treat event_maj relative_maj

keep if fm_ratio!=.

save "${data}/uni_by_major.dta", replace





