version 17 

*********** --------------------------------- *********** 
** Step 2.1:  Table 1: six year average statistics
*********** --------------------------------- *********** 

*** Data from extenral sources for Table 1 - Panels A-D
import excel "${data}/Descriptive_tabel.xlsx", sheet("Sheet2") firstrow clear

replace F_hS=F_hS/1000
replace M_hS = M_hS/1000
replace all_hS=all_hS/1000
replace f_test=f_test/1000
replace m_test=m_test/1000
replace all_test=all_test /1000
replace f_uni=f_uni/1000
replace m_uni=m_uni/1000
replace all_uni=all_uni/1000
replace f_privat_uni=f_privat_uni/1000
replace m_privat_uni=m_privat_uni/1000
replace all_privat_uni=all_privat_uni/1000

local avgSD "Average = r(mean) Sdv = r(sd)"
display "`myresults'" 

table (command) (result), ///
	command(`avgSD' : sum F_hS) ///
	command(`avgSD' : sum M_hS) ///
	command(`avgSD' : sum all_hS) ///
	command(`avgSD' : sum f_test) ///
	command(`avgSD' : sum m_test) ///
	command(`avgSD' : sum all_test) ///
	command(`avgSD' : sum f_uni) ///
	command(`avgSD' : sum m_uni) ///
	command(`avgSD' : sum all_uni) ///
	command(`avgSD' : sum f_privat_uni) ///
	command(`avgSD' : sum m_privat_uni) ///
	command(`avgSD' : sum all_privat_uni)

collect label levels command 1 "Female HS graduate" 2 "Male High School graduate" 3 "All High School Graduate" 4 "Female test taker" 5 "Male test taker" 6 "All test taker" 7 " Female admitted" 8 "Male admitted" 9 "All admitted" 10 "Female enrolled in Priv-IHE" 11 "Male enrolled in Priv-IHE" 12 "All enrolled in Priv-IHE", replace

collect label list result, all
collect style cell result[Average Sdv], nformat(%8.2f)
collect style cell result[], nformat(%6.2f) // same as above
collect style cell result[Sdv], sformat("(%s)")
collect style cell border_block, border(right, pattern(nil))

collect preview
collect export "${tables}/table_1/Tab1_Panels_A_D.docx", replace



* ---------------------------- ** ---------------------------- *
*use "${data}/kankor_all.dta", clear 

* ---------------------------- *
*** Panel E: Kankor participants
* ---------------------------- *

*sum test
*local total_mean = round(r(mean), .01)
*local total_SD = round(r(sd), .01)

* Perform the ttest and capture means and SDs for each gender
*ttest test, by(gender)

*local mean_female = round(r(mu_1), .01)
*local sd_female = round(r(sd_1), .01)
*local mean_male  = round(r(mu_2), .01)
*local sd_male = round(r(sd_2), .01)

* Calculate the ratio of mean_male to mean_female
*if r(mu_2) != 0 {
 *   local fm_ratio = r(mu_1) / r(mu_2)
*}
*else {
 *   local fm_ratio = "."
*}

* Open a new file for writing
*file open myfile using "${tables}/table_1/Tab1_Panel_E.csv", write replace

* Write the column names
*file write myfile "Mean_Female, SD_Female, Mean_Male, SD_Male, Total_Mean, Total_SD,  fm_ratio" _n

* Write the data
*file write myfile (`mean_female') "," (`sd_female') "," (`mean_male') "," (`sd_male') "," (`total_mean') "," (`total_SD') "," (`fm_ratio') _n

* Close the file
*file close myfile


* ---------------------------- *
*** Panel F: admitted to university 
* ---------------------------- *

*sum test if ihetype==3
*local total_mean = round(r(mean), .01)
*local total_SD = round(r(sd), .01)

* Perform the ttest and capture means and SDs for each gender
*ttest test if ihetype==3, by(gender)

*local mean_female = round(r(mu_1), .01)
*local sd_female = round(r(sd_1), .01)
*local mean_male = round(r(mu_2), .01)
*local sd_male = round(r(sd_2), .01)

* Calculate the ratio of mean_female to mean_male
* Ensure r(mu_2) is not zero before division to avoid division by zero errors
*if r(mu_2) != 0 {
 *   local fm_ratio = r(mu_1) / r(mu_2)
*}
*else {
 *   local fm_ratio = "."
*}

* Open a new file for writing
*file open myfile using "${tables}/table_1/Tab1_Panel_F.csv", write replace

* Write the column names
*file write myfile " Mean_Female, SD_Female, Mean_Male, SD_Male, Total_Mean, Total_SD, fm_ratio" _n

* Write the data
*file write myfile (`mean_female') "," (`sd_female') "," (`mean_male') "," (`sd_male') ","  (`total_mean') "," (`total_SD') "," (`fm_ratio') _n

* Close the file
*file close myfile





*********** ----------------------------------------- *********** 
** Step 2.2: Figure 1: Share of female applicants treated units
*********** ----------------------------------------- *********** 

use "${data}/uni_by_concentration.dta", clear // load the datset 

replace share_female= share_female*100
keep if everconcent==1

** share in pre-Treatment
bysort concent: egen pre_avg=mean(share_female) if relative<0
bysort concent: egen share_pre=max(pre_avg)
format share_pre %10.2f

** share in post-Treatment
bysort concent: egen post_avg=mean(share_female) if relative>=0
bysort concent: egen share_post=max(post_avg)
format share_post %10.2f

keep concent share_pre share_post uni
duplicates drop concent, force

twoway (scatter share_post share_pre, mlcolor(cranberry%30) mcolor(edkblue%60) msize(5pt) legend(label(1 "% female in concentration"))) (lfit share_post share_pre, clcolor(blue) clwidth(mthin) legend(label(2 "Fitted value"))), ytitle("% female enrolled {bf:post-quota}", size(small) xoffset(-1.5)) xtitle("% female enrolled {bf:pre-quota}", size(small) yoffset(-1.5)) xlabel(#4, labsize(vsmall)) ylabel(#4, labsize(vsmall) glwidth(thin)) yscale(lwidth(thin) lcolor(gs13%80)) xscale(lwidth(thin) lcolor(gs13%80)) graphregion(fcolor(white) lcolor(white)) plotregion(fcolor(lime%3)) legend(size(small) pos(4) ring(0) region(fcolor(none))) name(figure1, replace) 


graph save "figure1" "${figures}/shareFemale_treatedunits.gph", replace
graph export "${figures}/shareFemale_treatedunits.png", as(png) name("figure1") replace

