version 17 

**************************************************************************
** Step 2.3: Produce event study results 
**************************************************************************

use "${data}/uni_by_concentration.dta", clear // load the datset 


xtset concent year_kankor // declear data as panel 

set graphics off

** --------------------- **** --------------------- **
				* set local and global 

local borusyak21 "concent year_kankor event, allhorizons pretrend(3) autos fe(uni uni_year acdem_program maj_year) cluster(concent)"

local sunAbra "L*event F*event, vce(cluster concent) absorb(i.concent i.year_kankor i.uni##i.year_kankor i.acdem_program##i.year_kankor) cohort(event) control_cohort(nevertreated)"

local ols_var "F*event L*event, a(i.concent i.uni##i.year_kankor i.acdem_program##i.year_kankor i.year_kankor) cl(concent)"

global event_outlook "bjs sa_b#sa_v ols, 		stub_lag(tau# L#event L#event) stub_lead(pre# F#event F#event) plottype(scatter)		 ciplottype(rcap) together perturb(-0.1(0.05)0.1) noautolegend trimlead(3) 			 graph_opt(title("", size(medlarge))		 xtitle("Period since the treatment", size(vsmall) yoffset(-1) xoffset(10)) ytitle("estimates", size(small))				xlabel(-3(1)2) ylabel(#5, labsize(vsmall) glwidth(vthin) glcolor(gs15)) 			yscale(lwidth(medthick) lcolor(erose%20)) xscale(lwidth(medthick) lcolor(erose%20)) 	 	xline(0, lcolor(gs13) lpattern(dash)) yline(0, lcolor(navy) lwidth(thin)) 			 graphregion(color(white)) bgcolor(white) ylabel(, angle(horizontal))	legend(order(5 "OLS" 3 "Sun-Abraham (2020)" 1 "Borusyak et al. (2021)" ) rows(1) region(style(none)) pos(6))) lag_opt1(msymbol(Dh) color(orange)) lag_ci_opt1(color(orange)) 		lag_opt2(msymbol(Th) color(cranberry)) lag_ci_opt2(color(cranberry)) 	lag_opt3(msymbol(Oh) color(ebblue)) lag_ci_opt3(color(ebblue))"

* generate interaction terms to use for Borusyak et al. (2021)
gen uni_year=uni*year_kankor
gen maj_year=acdem_program*year_kankor


** ----------- ** ** ----------- ** ** ----------- ** 
* Outcome #1: share of female applicants
** ----------- ** ** ----------- ** ** ----------- ** 

preserve

// #1 Borusyak et al. (2021): Estimation with did_imputation of 
did_imputation share_female `borusyak21'

//  event_plot, default_look 
estimates store bjs // storing the estimates for later

// #2 Sun and Abraham (2020): Estimation with eventstudyinteract
sum event
gen nevertreated = everconcent==0 // dummy for the never-treated cohort
forvalues l = 0/2 {
	gen L`l'event = relative==`l'
}
forvalues l = 1/3 {
	gen F`l'event = relative==-`l'
}
drop F1event // normalize K=-1 to zero

eventstudyinteract share_female `sunAbra'

matrix sa_b = e(b_iw) // storing the estimates for later
matrix sa_v = e(V_iw)

// #2 OLS estimation
reghdfe share_female `ols_var'

estimates store ols // saving the estimates for later

// Combine all plots using the stored estimates
event_plot ${event_outlook}
	
graph export "${figures}/a_shareFem_quota.png", replace

restore


** ----------- ** ** ----------- ** ** ----------- ** 
* Outcome #2: share of female from low-SES background
** ----------- ** ** ----------- ** ** ----------- ** 

preserve
// #1 Borusyak et al. (2021): Estimation with did_imputation of 
did_imputation share_female_lowSE `borusyak21'

estimates store bjs // storing the estimates for later

// #2 Sun and Abraham (2020): Estimation with eventstudyinteract
sum event
gen nevertreated = everconcent==0 // dummy for the never-treated cohort
forvalues l = 0/2 {
	gen L`l'event = relative==`l'
}
forvalues l = 1/3 {
	gen F`l'event = relative==-`l'
}
drop F1event // normalize K=-1 to zero

eventstudyinteract share_female_lowSE `sunAbra'

matrix sa_b = e(b_iw) // storing the estimates for later
matrix sa_v = e(V_iw)

// #3 OLS estimation
reghdfe share_female_lowSE `ols_var'

estimates store ols // saving the estimates for later

// Combine all plots using the stored estimates

event_plot ${event_outlook}

graph export "${figures}/b_shareFem_quota_LSE.png", replace

restore

** ----------- ** ** ----------- ** ** ----------- ** 
* Outcome #3: Total average score
** ----------- ** ** ----------- ** ** ----------- ** 

preserve
// #1 Borusyak et al. (2021): Estimation with did_imputation of 
did_imputation avg_score `borusyak21'

estimates store bjs // storing the estimates for later

// #2 Sun and Abraham (2020): Estimation with eventstudyinteract
sum event
gen nevertreated = everconcent==0 // dummy for the never-treated cohort
forvalues l = 0/2 {
	gen L`l'event = relative==`l'
}
forvalues l = 1/3 {
	gen F`l'event = relative==-`l'
}
drop F1event // normalize K=-1 to zero

eventstudyinteract avg_score `sunAbra'

matrix sa_b = e(b_iw) // storing the estimates for later
matrix sa_v = e(V_iw)

// #5 OLS estimation 
reghdfe avg_score `ols_var'

estimates store ols // saving the estimates for later

// Combine all plots using the stored estimates
event_plot ${event_outlook}
	
graph export "${figures}/g_All_avgSc_quota.png", replace

restore 

** ----------- ** ** ----------- ** ** ----------- ** 
* Outcome #4: Female applicants average score 
** ----------- ** ** ----------- ** ** ----------- ** 
preserve

// #1 Borusyak et al. (2021): Estimation with did_imputation of 
did_imputation female_avg_score `borusyak21'

estimates store bjs // storing the estimates for later

// #2 Sun and Abraham (2020): Estimation with eventstudyinteract
sum event
gen nevertreated = everconcent==0 // dummy for the never-treated cohort
forvalues l = 0/2 {
	gen L`l'event = relative==`l'
}
forvalues l = 1/3 {
	gen F`l'event = relative==-`l'
}
drop F1event // normalize K=-1 to zero

eventstudyinteract female_avg_score `sunAbra'

matrix sa_b = e(b_iw) // storing the estimates for later
matrix sa_v = e(V_iw)

// #5 OLS estimation
reghdfe female_avg_score `ols_var'

estimates store ols // saving the estimates for later

// Combine all plots using the stored estimates
event_plot ${event_outlook}
	
graph export "${figures}/c_Fem_avgSc_quota.png", replace

restore

** --------------------------- ** ** --------------------------- ** 
* Outcome #5: Average score for female applicants from High SES 
** --------------------------- ** ** --------------------------- ** 

preserve

// #1 Borusyak et al. (2021): Estimation with did_imputation of 
did_imputation fem_avg_score_HSES `borusyak21'

estimates store bjs // storing the estimates for later

// # 4 Sun and Abraham (2020): Estimation with eventstudyinteract
sum event
gen nevertreated = everconcent==0 // dummy for the never-treated cohort
forvalues l = 0/2 {
	gen L`l'event = relative==`l'
}
forvalues l = 1/3 {
	gen F`l'event = relative==-`l'
}
drop F1event // normalize K=-1 to zero

eventstudyinteract fem_avg_score_HSES `sunAbra'

matrix sa_b = e(b_iw) // storing the estimates for later
matrix sa_v = e(V_iw)

// #5 OLS estimation
reghdfe fem_avg_score_HSES `ols_var'

estimates store ols // saving the estimates for later

// Combine all plots using the stored estimates
event_plot ${event_outlook}
	
graph export "${figures}/d_Fem_avgSc_HSES_quota.png", replace

restore

** --------------------------- ** ** --------------------------- **  
* Outcome #6: Average score for female applicants from low SES 
** --------------------------- ** ** --------------------------- ** 

preserve

// #1 Borusyak et al. (2021): Estimation with did_imputation of 
did_imputation fem_avg_score_LSES `borusyak21'

estimates store bjs // storing the estimates for later

// #2 Sun and Abraham (2020): Estimation with eventstudyinteract
sum event
gen nevertreated = everconcent==0 // dummy for the never-treated cohort
forvalues l = 0/2 {
	gen L`l'event = relative==`l'
}
forvalues l = 1/3 {
	gen F`l'event = relative==-`l'
}
drop F1event // normalize K=-1 to zero

eventstudyinteract fem_avg_score_LSES `sunAbra'
matrix sa_b = e(b_iw) // storing the estimates for later
matrix sa_v = e(V_iw)

// #3 OLS estimation 
reghdfe fem_avg_score_LSES `ols_var'

estimates store ols // saving the estimates for later

// Combine all plots using the stored estimates
event_plot ${event_outlook}
	
graph export "${figures}/e_Fem_avgSc_LSES_quota.png", replace

restore


** --------------------------- ** ** --------------------------- **  
* Outcome #7: Average score for male applicants
** --------------------------- ** ** --------------------------- ** 

preserve

// #1 Borusyak et al. (2021): Estimation with did_imputation of 
did_imputation male_avg_score `borusyak21'

estimates store bjs // storing the estimates for later

// #2 Sun and Abraham (2020): Estimation with eventstudyinteract
sum event
gen nevertreated = everconcent==0 // dummy for the never-treated cohort
forvalues l = 0/2 {
	gen L`l'event = relative==`l'
}
forvalues l = 1/3 {
	gen F`l'event = relative==-`l'
}
drop F1event // normalize K=-1 to zero

eventstudyinteract male_avg_score `sunAbra'

matrix sa_b = e(b_iw) // storing the estimates for later
matrix sa_v = e(V_iw)

// #3 OLS estimation 
reghdfe male_avg_score `ols_var'

estimates store ols // saving the estimates for later

// Combine all plots using the stored estimates
event_plot ${event_outlook}

graph export "${figures}/f_Male_avgSc_quota.png", replace

restore

** --------------------------- ** ** --------------------------- **  
* Outcome #8: Female to male applicants Average score ratio
** --------------------------- ** ** --------------------------- ** 

preserve

// #1 Borusyak et al. (2021): Estimation with did_imputation of 
did_imputation fm_ratio `borusyak21'

estimates store bjs // storing the estimates for later

// #2 Sun and Abraham (2020): Estimation with eventstudyinteract
sum event
gen nevertreated = everconcent==0 // dummy for the never-treated cohort
forvalues l = 0/2 {
	gen L`l'event = relative==`l'
}
forvalues l = 1/3 {
	gen F`l'event = relative==-`l'
}
drop F1event // normalize K=-1 to zero
eventstudyinteract fm_ratio `sunAbra'

matrix sa_b = e(b_iw) // storing the estimates for later
matrix sa_v = e(V_iw)

// #3 OLS estimation
reghdfe fm_ratio `ols_var'

estimates store ols // saving the estimates for later

// Combine all plots using the stored estimates
event_plot ${event_outlook}
	
graph export "${figures}/h_fem_mal_score_quota.png", replace
restore

set graphics on
