version 17 

use "${data}/uni_by_concentration.dta", clear // load the datset 

xtset concent year_kankor // declear data as panel 

set graphics off // turn off graphics

lab var event "cohort identifier"

**************************************************************************
** Step 2.3.1: Table 2: Main regression results 
**************************************************************************

				* ------------- * * ------------- *  
				* 	Set global and local macros	  *
				* ------------- * * ------------- * 
				
local dep_vars "share_female share_female_lowSE avg_score female_avg_score fem_avg_score_HSES fem_avg_score_LSES male_avg_score fm_ratio" // Table 2 

global ols_vars "i.gp, a(i.concent i.uni##i.year_kankor i.acdem_program##i.year_kankor i.year_kankor) cl(concent)"

global cells_ols "cells(b(star fmt(2)) se(par fmt(2)))  starlevels(* 0.10 ** 0.05 *** 0.01)    stats(r2 N m_share_female m_share_female_lowSE m_avg_score m_female_avg_score m_fem_avg_score_HSES m_fem_avg_score_LSES m_male_avg_score m_fm_ratio,   fmt(2 2 2 2 2 2 2 2 2 2)  labels("R-squared" "N" "Mean % fem" "%fem-lses" "avg_sc" "fem_scr" "fem_hses_scor" "fem_lses_scor" "male_scor" "fm_ratio"))  keep(1.gp)  varlabels(1.gp "treat")   collabels(none)   mlabels("% fem" "%fem-lses" "avg_sc" "fem_scr" "fem_hses_scor" "fem_lses_scor" "male_scor" "fm_ratio")   title("Table #. OLS - Effects of gender admission quota on outcomes at concentration level")   modelwidth(15)   addnote("Note:")"


global cells_Borusyak "cells(b(star fmt(2)) se(par fmt(2))) starlevels(* 0.10 ** 0.05 *** 0.01)  stats(r2 N m_share_female m_share_female_lowSE m_avg_score m_female_avg_score m_fem_avg_score_HSES m_fem_avg_score_LSES m_male_avg_score m_fm_ratio, fmt(2 2 2 2 2 2 2 2 2 2)  labels("R-squared" "N" "Mean % fem" "%fem-lses" "avg_sc" "fem_scr" "fem_hses_scor" "fem_lses_scor" "male_scor" "fm_ratio"))  keep(tau)  varlabels(tau "treat") collabels(none)  mlabels("% fem" "%fem-lses" "avg_sc" "fem_scr" "fem_hses_scor" "fem_lses_scor" "male_scor" "fm_ratio") title("Table #. Effects of gender admission quota - Borusyak ") modelwidth(11) addnote("Note:")"

global Borusyak_vars "concent year_kankor event if stu_samp==1, autos fe(uni uni_year acdem_program maj_year)"
 

global SunAbraham_vars "L*event F*event, vce(cluster concent) absorb(i.concent i.year_kankor i.uni##i.year_kankor i.acdem_program##i.year_kankor) cohort(event) control_cohort(nevertreated)"
 
 
gen uni_year=uni*year_kankor
gen maj_year=acdem_program*year_kankor


* ------------- *
** #1 OLS estimator 

foreach var of local dep_vars {
eststo : quietly reghdfe `var' ${ols_vars}
    sum `var' if e(sample) & everconcent==0
    scalar evnt1 = r(mean)
	estadd scalar m_`var' = evnt1
}

gen stu_samp=1 if _est_est1==1

esttab using "${tables}/Tab2_OLS.tex", replace ${cells_ols}

/*** No need to run but just if interested in coefplot -- copy and paste and change file name to produce for subsequent models (e.g., Tabele 3 and 4)

*** coefplot for access 
 
global access_coef "keep(1.gp) yscale(off lwidth(medthick) lcolor(erose%20)) ytick(none) xscale(lwidth(thick) lcolor(ebblue%20)) xtick(#5) xline(0, lwidth(thin) lcolor(gs12)) level(95) rename(1.gp="Treatment") legend(cols(1) rowgap(15) size(small) position(left)  region(fcolor(none) lcolor(none))) 	xlabel(#5, labsize(vsmall)) ylabel(, labsize(small)) 	p1(msymbol(d) ciopts(lwidth(thick) lcolor(teal%40)) msize(medlarge) color(teal) lwidth(thin) label("Share of female" "students")) p2(msymbol(o) ciopts(lwidth(thick) lcolor(ebblue%40)) msize(medlarge) color(ebblue) label("Share of female" "students - low-SES")) p3(msymbol(t) ciopts(lwidth(thick) lcolor(maroon%40)) msize(medlarge) color(maroon) label("female to male" "avg. score ratio")) graphregion(fcolor(white)) ciopts(lwidth(thin)) xsize(1) ysize(1) plotregion(fcolor(gray%8))"

  coefplot est1 est2 est8, ${access_coef} // export coefplot 
graph export "${figures}/ATT_1_access.png", as(png) name("Graph") replace


*** coefplot for score 

global score_coef "keep(1.gp) yscale(off lwidth(medthick) lcolor(erose%20)) ytick(none) xscale(lwidth(thick) lcolor(ebblue%20)) xtick(#5) xline(0, lwidth(thin) lcolor(gs12)) level(95) rename(1.gp="Treatment") legend(cols(1) rowgap(10) size(small) position(left)  region(fcolor(none) lcolor(none))) xlabel(#5, labsize(vsmall)) ylabel(, labsize(small)) p1(msymbol(d) ciopts(lwidth(thick) lcolor(blue%30)) msize(medlarge) color(blue%70) lwidth(thin) label("total avg. score")) p2(msymbol(s) ciopts(lwidth(thick) lcolor(orange_red%40)) msize(medlarge) color(orange_red) label("female avg. score")) p3(msymbol(t) ciopts(lwidth(thick) lcolor(cranberry%40)) msize(medlarge) color(cranberry) label("high-SES female" "avg. score")) p4(msymbol(o) ciopts(lwidth(thick) lcolor(olive%40)) msize(medlarge) color(olive) label("low-SES female " "avg. score")) p5(msymbol(s) ciopts(lwidth(thick) lcolor(emerald%40)) msize(medlarge) color(emerald) label("male avg. score")) graphregion(fcolor(white)) ciopts(lwidth(thin)) xsize(1) ysize(1) plotregion(fcolor(gray%8))"

coefplot est3 est4 est5 est6 est7,${score_coef} // export coefplot 

graph export "${figures}/ATT_2_scor.png", as(png) name("Graph") replace
*/  

estimate clear


* ------------- *
** #2 Borusyak et al. (2021): Estimation with did_imputation of

foreach var of local dep_vars {
eststo : quietly did_imputation `var' ${Borusyak_vars}
    sum `var' if e(sample) & everconcent==0
    scalar evnt1 = r(mean)
	estadd scalar m_`var' = evnt1
}

esttab using "${tables}/Tab2_Borusyak.tex", replace ${cells_Borusyak}

* ------------- *
** #3 Sun & Abraham Estimator

preserve
sum event
gen nevertreated = everconcent==0 // dummy for the never-treated cohort
forvalues l = 0/2 {
	gen L`l'event = relative==`l'
}
forvalues l = 1/3 {
	gen F`l'event = relative==-`l'
}
drop F1event // normalize K=-1 to zero

// Define a new dataset to store the results
tempname results_data
postfile `results_data' depvar estimate se p_value using results_data.dta, replace

local i = 1

foreach var of local dep_vars {
   quietly  eventstudyinteract `var' ${SunAbraham_vars}

    matrix b = e(b_iw)
    matrix V = e(V_iw)
    ereturn post b V
    lincom (L0event + L1event + L2event)/3

    scalar b_ht = r(estimate)
    scalar se_ht = r(se)
	scalar p_value =r(p)

    // Store the estimates and standard errors in the dataset
    post `results_data' (`i') (b_ht) (se_ht) (p_value)

    local ++i
}

// Close the postfile
postclose `results_data'

// Load the dataset and display the table
use results_data.dta, clear
list

// Export the dataset to a CSV file
outsheet using "${tables}/Tab2_sunAbraham.csv", comma replace
  
  estimates clear
  
restore



**************************************************************************
** Step 2.3.2: Table 3: Heterogeneous results  Panel A & B 
**************************************************************************

				* ------------- * * ------------- *  
				* 	Set global and local macros	  *
				* ------------- * * ------------- * 

local dep_vars "share_female share_female_lowSE avg_score female_avg_score fem_avg_score_HSES fem_avg_score_LSES male_avg_score fm_ratio" // Table 3 outcomes



global ceelm_ols_vars "i.gp if ceelm==1, a(i.concent i.uni##i.year_kankor i.acdem_program##i.year_kankor i.year_kankor) cl(concent)"

global no_ceelm_ols_vars "i.gp if ceelm==0, a(i.concent i.uni##i.year_kankor i.acdem_program##i.year_kankor i.year_kankor) cl(concent)"

global hetro_cells_OLs "cells(b(star fmt(2)) se(par fmt(2))) starlevels(* 0.10 ** 0.05 *** 0.01)   stats(r2 N m_share_female m_share_female_lowSE m_avg_score m_female_avg_score m_fem_avg_score_HSES m_fem_avg_score_LSES m_male_avg_score m_fm_ratio,   fmt(2 2 2 2 2 2 2 2 2 2)   labels("R-squared" "N" "Mean % fem" "%fem-lses" "avg_sc" "fem_scr" "fem_hses_scor" "fem_lses_scor" "male_scor" "fm_ratio"))  keep(1.gp)  varlabels(1.gp "treat")  collabels(none) mlabels("% fem" "%fem-lses" "avg_sc" "fem_scr" "fem_hses_scor" "fem_lses_scor" "male_scor" "fm_ratio")  title("Table 3. OLS - Hetro effects of gender quota on outcomes at concentration level ")  modelwidth(11)  addnote("")"


global ceelm_Borusyak_vars "concent year_kankor event if ceelm==1 & het_sam1==1, autos fe(uni uni_year acdem_program maj_year)"

global non_ceelm_Borusyak_vars "concent year_kankor event if ceelm==0 & het_sam2==1, autos fe(uni uni_year acdem_program maj_year)"

global hetro_cells_Borusyak "cells(b(star fmt(2)) se(par fmt(2)))   starlevels(* 0.10 ** 0.05 *** 0.01) stats(r2 N m_share_female m_share_female_lowSE m_avg_score m_female_avg_score m_fem_avg_score_HSES m_fem_avg_score_LSES m_male_avg_score m_fm_ratio,  fmt(2 2 2 2 2 2 2 2 2 2) labels("R-squared" "N" "Mean % fem" "%fem-lses" "avg_sc" "fem_scr" "fem_hses_scor" "fem_lses_scor" "male_scor" "fm_ratio")) keep(tau) varlabels(tau "treat") collabels(none) mlabels("% fem" "%fem-lses" "avg_sc" "fem_scr" "fem_hses_scor" "fem_lses_scor" "male_scor" "fm_ratio")  title("Table 3. Hetro effects of gender quota on outcomes - Borusyak")  modelwidth(11) addnote("")"



global ceelm_SunAbraham "L*event F*event if ceelm==1, vce(cluster concent) absorb(i.concent i.year_kankor i.uni##i.year_kankor i.acdem_program##i.year_kankor) cohort(event) control_cohort(nevertreated)"

global non_ceelm_SunAbraham "L*event F*event if ceelm==0, vce(cluster concent) absorb(i.concent i.year_kankor i.uni##i.year_kankor i.acdem_program##i.year_kankor) cohort(event) control_cohort(nevertreated)"



******		----------------- ****************----------------- 	*****
*		Table 3: Panel A: CEELM - Competitive Academic Programs 

* ------------- *
** #1 OLS estimator 

foreach var of local dep_vars {
eststo : quietly reghdfe `var' ${ceelm_ols_vars}
    sum `var' if e(sample) & everconcent==0
    scalar evnt1 = r(mean)
	estadd scalar m_`var' = evnt1
}

gen het_sam1=1 if _est_est1==1 // use for Borusyak

esttab using "${tables}/Tab3_panel_A_OLS.tex", replace ${hetro_cells_OLs}

estimates clear


* ------------- *
** #2 Borusyak et al. (2021): Estimation with did_imputation of

foreach var of local dep_vars {
eststo : quietly did_imputation `var' ${ceelm_Borusyak_vars}
    sum `var' if e(sample) & everconcent==0
    scalar evnt1 = r(mean)
	estadd scalar m_`var' = evnt1
}

esttab using "${tables}/Tab3_panel_A_Borusyak.tex", replace ${hetro_cells_Borusyak}

estimates clear

* ------------- *
** #3 Sun & Abraham Estimator

preserve
sum event
gen nevertreated = everconcent==0 // dummy for the never-treated cohort
forvalues l = 0/2 {
	gen L`l'event = relative==`l'
}
forvalues l = 1/3 {
	gen F`l'event = relative==-`l'
}
drop F1event // normalize K=-1 to zero

// Define a new dataset to store the results
tempname results_data
postfile `results_data' depvar estimate se p_value using results_data.dta, replace

local i = 1
foreach var of local dep_vars {
    quietly eventstudyinteract `var' ${ceelm_SunAbraham}

    matrix b = e(b_iw)
    matrix V = e(V_iw)
    ereturn post b V
    lincom (L0event + L1event + L2event)/3

    scalar b_ht = r(estimate)
    scalar se_ht = r(se)
	scalar p_value =r(p)

    // Store the estimates and standard errors in the dataset
    post `results_data' (`i') (b_ht) (se_ht) (p_value)

    local ++i
}

// Close the postfile
postclose `results_data'

// Load the dataset and display the table
use results_data.dta, clear
list

// Export the dataset to a CSV file
outsheet using "${tables}/Tab3_panel_A_SunAbraham.csv", comma replace
  
  estimates clear
restore




******		----------------- ****************----------------- 	
*		Panle 3: Panel B: non-CEELM - non-competitive Academic Programs 


* ------------- * 
** #1 OLS estimator 

foreach var of local dep_vars {
eststo : quietly reghdfe `var' ${no_ceelm_ols_vars}
    sum `var' if e(sample) & everconcent==0
    scalar evnt1 = r(mean)
	estadd scalar m_`var' = evnt1
}

gen het_sam2=1 if _est_est1==1 // use for Borusyak

esttab using "${tables}/Tab3_panel_B_OLS.tex", replace ${hetro_cells_OLs}

estimates clear


* ------------- * 
** #2 Borusyak et al. (2021): Estimation with did_imputation of

foreach var of local dep_vars {
eststo : quietly did_imputation `var' ${non_ceelm_Borusyak_vars}
    sum `var' if e(sample) & everconcent==0
    scalar evnt1 = r(mean)
	estadd scalar m_`var' = evnt1
}

esttab using "${tables}/Tab3_panel_B_Borusyak.tex", replace ${hetro_cells_Borusyak}

estimates clear

* ------------- * 
** #3 Sun & Abraham Estimator

preserve
sum event
gen nevertreated = everconcent==0 // dummy for the never-treated cohort
forvalues l = 0/2 {
	gen L`l'event = relative==`l'
}
forvalues l = 1/3 {
	gen F`l'event = relative==-`l'
}
drop F1event // normalize K=-1 to zero

// Define a new dataset to store the results
tempname results_data
postfile `results_data' depvar estimate se p_value using results_data.dta, replace

local i = 1
foreach var of local dep_vars {
    quietly eventstudyinteract `var' ${non_ceelm_SunAbraham}

    matrix b = e(b_iw)
    matrix V = e(V_iw)
    ereturn post b V
    lincom (L0event + L1event + L2event)/3

    scalar b_ht = r(estimate)
    scalar se_ht = r(se)
	scalar p_value =r(p)

    // Store the estimates and standard errors in the dataset
    post `results_data' (`i') (b_ht) (se_ht) (p_value)

    local ++i
}

// Close the postfile
postclose `results_data'

// Load the dataset and display the table
use results_data.dta, clear
list

// Export the dataset to a CSV file
outsheet using "${tables}/Tab3_panel_B_SunAbraham.csv", comma replace
  
  estimates clear
restore




**************************************************************************
** Step 2.3.3: Table 4: Heterogeneous results - male applicants
**************************************************************************

				* ------------- * * ------------- *  
				* 	Set global and local macros	  *
				* ------------- * * ------------- * 
				
local dep_vars_male "share_male share_male_lowSE ma_avg_score_HSES ma_avg_score_LSES" // Table 4  outcomes


global ceelm_male_ols_vars "i.gp if ceelm==1, a(i.concent i.uni##i.year_kankor i.acdem_program##i.year_kankor i.year_kankor) cl(concent)"

global no_male_ceelm_ols_vars "i.gp if ceelm==0, a(i.concent i.uni##i.year_kankor i.acdem_program##i.year_kankor i.year_kankor) cl(concent)"

global hetro_male_cells_OLs "cells(b(star fmt(2)) se(par fmt(2))) starlevels(* 0.10 ** 0.05 *** 0.01) stats(r2 N m_share_male m_share_male_lowSE m_ma_avg_score_HSES m_ma_avg_score_LSES,   fmt(2 2 2 2 2 2 2 2 2 2) labels("R-squared" "N" "Mean % male" "%male-lses" "male_HSES_score" "male_LSES_score"))  keep(1.gp) varlabels(1.gp "treat")  collabels(none) mlabels("% male" "%male-lses" "avg_M_hses" "avg_M_Lses")  title("Table 4. OLS - Hetro effects of gender quota on outcomes at concentration level ")  modelwidth(11)  addnote("")"



global ceelm_male_Borusyak_vars "concent year_kankor event if ceelm==1 & het_male_ceelm==1, autos fe(uni uni_year acdem_program maj_year)"

global non_male_ceelm_Borusyak_vars "concent year_kankor event if ceelm==0 & het_male_nonCeelm==1, autos fe(uni uni_year acdem_program maj_year)"


global hetro_male_cells_Borusyak "cells(b(star fmt(2)) se(par fmt(2)))  starlevels(* 0.10 ** 0.05 *** 0.01) stats(r2 N m_share_male m_share_male_lowSE m_ma_avg_score_HSES m_ma_avg_score_LSES,   fmt(2 2 2 2 2 2 2 2 2 2) labels("R-squared" "N" "Mean % male" "%male-lses" "avg_score_HSES" "avg_score_LSES"))  keep(tau) varlabels(tau "treat") collabels(none) mlabels("% male" "%male-lses" "avg_M_hses" "avg_M_Lses")  title("Table 4. Hetro effects of gender quota on outcomes at concentration level - Borusyak et. al.")  modelwidth(11)  addnote("")"




global ceelm_male_SunAbraham "L*event F*event if ceelm==1, vce(cluster concent) absorb(i.concent i.year_kankor i.uni##i.year_kankor i.acdem_program##i.year_kankor) cohort(event) control_cohort(nevertreated)"

global non_male_ceelm_SunAbraham "L*event F*event if ceelm==0, vce(cluster concent) absorb(i.concent i.year_kankor i.uni##i.year_kankor i.acdem_program##i.year_kankor) cohort(event) control_cohort(nevertreated)"

******		----------------- ****************----------------- 	*****
*		Table 4: Panel A: CEELM Male - Competitive Academic Programs 


* ------------- * 
** #1 OLS estimator 

foreach var of local dep_vars_male {
eststo : quietly reghdfe `var' ${ceelm_male_ols_vars}
    sum `var' if e(sample) & everconcent==0
    scalar evnt1 = r(mean)
	estadd scalar m_`var' = evnt1
}

gen het_male_ceelm=1 if _est_est1==1 // use for Borusyak

esttab using "${tables}/Tab4_panel_A_OLS.tex", replace ${hetro_male_cells_OLs}

estimates clear


* ------------- * 
** #2 Borusyak et al. (2021): Estimation with did_imputation of

foreach var of local dep_vars_male {
eststo : quietly did_imputation `var' ${ceelm_male_Borusyak_vars}
    sum `var' if e(sample) & everconcent==0
    scalar evnt1 = r(mean)
	estadd scalar m_`var' = evnt1
}

esttab using "${tables}/Tab4_panel_A_Borusyak.tex", replace ${hetro_male_cells_Borusyak}

estimates clear

* ------------- *
** #3 Sun & Abraham Estimator

preserve
sum event
gen nevertreated = everconcent==0 // dummy for the never-treated cohort
forvalues l = 0/2 {
	gen L`l'event = relative==`l'
}
forvalues l = 1/3 {
	gen F`l'event = relative==-`l'
}
drop F1event // normalize K=-1 to zero

// Define a new dataset to store the results
tempname results_data
postfile `results_data' depvar estimate se p_value using results_data.dta, replace

local i = 1
foreach var of local dep_vars_male {
    quietly eventstudyinteract `var' ${ceelm_male_SunAbraham}

    matrix b = e(b_iw)
    matrix V = e(V_iw)
    ereturn post b V
    lincom (L0event + L1event + L2event)/3

    scalar b_ht = r(estimate)
    scalar se_ht = r(se)
	scalar p_value =r(p)

    // Store the estimates and standard errors in the dataset
    post `results_data' (`i') (b_ht) (se_ht) (p_value)

    local ++i
}

// Close the postfile
postclose `results_data'

// Load the dataset and display the table
use results_data.dta, clear
list

// Export the dataset to a CSV file
outsheet using "${tables}/Tab4_panel_A_SunAbraham.csv", comma replace
  
  estimates clear
restore


******		----------------- ****************----------------- 	
*		Panle 4: Panel B: non-CEELM Male - non-competitive Academic Programs 


* ------------- *
** #1 OLS estimator 

foreach var of local dep_vars_male {
eststo : quietly reghdfe `var' ${no_male_ceelm_ols_vars}
    sum `var' if e(sample) & everconcent==0
    scalar evnt1 = r(mean)
	estadd scalar m_`var' = evnt1
}

gen het_male_nonCeelm=1 if _est_est1==1 // use for Borusyak

esttab using "${tables}/Tab4_panel_B_OLS.tex", replace ${hetro_male_cells_OLs}

estimates clear

* ------------- *
** #2 Borusyak et al. (2021): Estimation with did_imputation of

foreach var of local dep_vars_male {
eststo : quietly did_imputation `var' ${non_male_ceelm_Borusyak_vars}
    sum `var' if e(sample) & everconcent==0
    scalar evnt1 = r(mean)
	estadd scalar m_`var' = evnt1
}

esttab using "${tables}/Tab4_panel_B_Borusyak.tex", replace ${hetro_male_cells_Borusyak}

estimates clear

* ------------- *
** #3 Sun & Abraham Estimator

preserve
sum event
gen nevertreated = everconcent==0 // dummy for the never-treated cohort
forvalues l = 0/2 {
	gen L`l'event = relative==`l'
}
forvalues l = 1/3 {
	gen F`l'event = relative==-`l'
}
drop F1event // normalize K=-1 to zero

// Define a new dataset to store the results
tempname results_data
postfile `results_data' depvar estimate se p_value using results_data.dta, replace

local i = 1
foreach var of local dep_vars_male {
    quietly eventstudyinteract `var' ${non_male_ceelm_SunAbraham}

    matrix b = e(b_iw)
    matrix V = e(V_iw)
    ereturn post b V
    lincom (L0event + L1event + L2event)/3

    scalar b_ht = r(estimate)
    scalar se_ht = r(se)
	scalar p_value =r(p)

    // Store the estimates and standard errors in the dataset
    post `results_data' (`i') (b_ht) (se_ht) (p_value)

    local ++i
}

// Close the postfile
postclose `results_data'

// Load the dataset and display the table
use results_data.dta, clear
list

// Export the dataset to a CSV file
outsheet using "${tables}/Tab4_panel_B_SunAbraham.csv", comma replace
  
  estimates clear
restore

