
version 17 

use "${data}/uni_by_concentration.dta", clear // load the datset 


******** 	--------------------------------	**********
* Step 3.1: Tab A3.1 : List of universities
******** 	--------------------------------	**********
preserve 

duplicates drop uni concent, force 
gen one=1
bysort uni: egen num_concent=sum(one)
bysort uni: egen num_Treated=sum(one) if everconcent==1
bysort uni: egen treat_concet=max(num_Treated)
replace treat_concet=0 if treat_concet==.

duplicates drop uni, force


* Calculate the total for each variable and define it as scalar
egen total_num_concent = sum(num_concent)
su total_num_concent
scalar Total=r(mean)

egen total_treat_concet = sum(treat_concet)
su total_treat_concet
scalar Treat_units=r(mean)

decode uni, generate(uni_1)

* Append a new observation
set obs `=_N+1'
replace uni_1 = "Total" in 38

replace num_concent=scalar(Total) if num_concent == . // replace missing value for the new observation with stored scalar
replace treat_concet=scalar(Treat_units) if treat_concet == .   // replace missing value for the new observation with stored scalar

gen rank=_n

bysort uni_1: gen share_treated=(treat_concet*100) / num_concent

sort rank
order uni_1 num_concent share_treated

keep uni_1 num_concent share_treated

duplicates drop uni_1, force

export delimited using "${tables}/appendix/Table_A3.1.csv", replace

restore 



******** 		--------------------------------			**********
*	Step 3.2: TabA3.2: List of academic programs
******** 		--------------------------------			**********
preserve 

duplicates drop acdem_program concent, force 
gen one=1
bysort acdem_program: egen num_concent=sum(one)
bysort acdem_program: egen num_Treated=sum(one) if everconcent==1
bysort acdem_program: egen treat_concet=max(num_Treated)
replace treat_concet=0 if treat_concet==.

duplicates drop acdem_program, force

* Calculate the total for each variable and define it as scalar
egen total_num_concent = sum(num_concent)
su total_num_concent
scalar Total=r(mean)

egen total_treat_concet = sum(treat_concet)
su total_treat_concet
scalar Treat_units=r(mean)

decode acdem_program, generate(acdem_program_1)

drop if acdem_program==.

* Append a new observation to add the Total values
set obs `=_N+1'
replace acdem_program_1 = "Total" in 18

replace num_concent=scalar(Total) if num_concent == . // replace missing value for the new observation with stored scalar
replace treat_concet=scalar(Treat_units) if treat_concet == . // replace missing value for the new observation with stored scalar

gen rank=_n

bysort acdem_program_1: gen share_treated=(treat_concet*100) / num_concent

sort rank
order acdem_program_1 num_concent share_treated

keep acdem_program_1 num_concent share_treated

export delimited using "${tables}/appendix/Table_A3.2.csv", replace

restore 


******** 	-------------------------------------	**********
*	Step 3.3: Tab B1.1: Yearly stats
******** 	-------------------------------------	**********
*preserve

*The following lines use kankor data, thus are commented out

*use "${data}/kankor_all.dta", clear  // load dataset

** # of concentrations per year 
*egen con_year=tag(concent year_kankor)
*bysort year_kankor: egen num_concent=sum(con_year)
*lab var num_concent "number of concentrations per year"

* share concent ever treated per year
*egen con_year_treat=tag(concent year_kankor everconcent)
*bysort year_kankor: egen n_t=sum(con_year_treat) if everconcent==1
*bysort year_kankor: egen num_treat_con=max(n_t)
*gen share_con_treat=(num_treat_con*100)/num_concent

* all test takers
*gen one=1
*bysort year_kankor: egen t_testTaker=sum(one)
*lab var t_testTaker "yearly total test takers"
*bysort year_kankor: gen testTaker000= (t_testTaker/1000)

* share female
*bysort year_kankor gender: egen t_tak_fem=sum(one)
*bysort year_kankor: gen f_t=t_tak_fem if gender==1
*bysort year_kankor: gen sh_fem=(t_tak_fem*100)/t_testTaker
*bysort year_kankor: egen fem_share=max(sh_fem) if gender==1
*bysort year_kankor: egen share_fem=max(fem_share)
*lab var share_fem "share of female test takers per year"

* yearly average score
*bysort year_kankor: egen avg_score=mean(test)
*lab var avg_score "yearly average score"

* share of Test Taker admitted to 4-year public IHEs
*bysort year_kankor ihetype: egen t_tak_ad=sum(one)
*bysort year_kankor: gen t_admi=t_tak_ad if ihetype==3 // identify those only admitted to public IHEs
*bysort year_kankor: gen sh_ad=(t_admi*100)/t_testTaker // create the share of admitted
*bysort year_kankor: egen adm_share=max(sh_ad) if ihetype==3
*bysort year_kankor: egen share_4yr_ad=max(adm_share) // populated for the whole column
*lab var share_4yr_ad "share of test takers marticualted into public 4-years IHEs"

* share of female test taker admitted to 4-year public IHEs
*bysort year_kankor ihetype gender: egen t_tak_ad_f=sum(one)
*bysort year_kankor: gen t_admi_f=t_tak_ad_f if ihetype==3 & gender==1 // identify those only admitted Female to public IHEs
*bysort year_kankor: gen sh_ad_fem=(t_admi_f*100)/t_admi // create the share of female admitted
*bysort year_kankor: egen adm_share_fem=max(sh_ad_fem) if ihetype==3 & gender==1
*bysort year_kankor: egen fem_sh4yr_ad=max(adm_share_fem) // populated for the whole column
*lab var fem_sh4yr_ad "share of Female out of marticualted applicants into public 4-years IHEs"




*keep year_kankor num_concent share_con_treat testTaker000 share_fem share_4yr_ad fem_sh4yr_ad avg_score
*order year_kankor num_concent share_con_treat testTaker000 share_fem share_4yr_ad fem_sh4yr_ad avg_score

*duplicates drop year_kankor, force

*drop if year_kankor==.

*outsheet using "${tables}/appendix/Tab_B1_1.csv", comma replace
*restore




******** 	-------------------------------------	**********
* Step 3.4: Tab B2.1: Pre- and post-2016 score distribution by treatment status
******** 	-------------------------------------	**********

preserve 

* Process for year_kankor < 2016
keep if year_kankor < 2016
collapse (mean) female_avg_score female_min_score female_max_score male_avg_score male_min_score male_max_score, by(everconcent) // collopase by treatment status pre-2016
outsheet using "${tables}/appendix/TabelB2_1_pre_2016.csv", comma replace
restore

preserve // Preserve the original dataset in memory
* Process for year_kankor >= 2016
keep if year_kankor >= 2016 
collapse (mean) female_avg_score female_min_score female_max_score male_avg_score male_min_score male_max_score, by(everconcent) // collopase by treatment status post-2016

outsheet using "${tables}/appendix/TabelB2_1_post_2016.csv", comma replace
restore



******** 	----------------------------------------	**********
*	Step 3.5: Tab_B3.1: Pre-2016 differences by treatment status
******** 	----------------------------------------	**********

preserve
keep if year_kankor<2016

label define value 0 "control" 1 "treated"
label values everconcent value


local diff_eq "Controal = r(mu_1) Treated = r(mu_2) Difference = (r(mu_2)-r(mu_1)) pvalue = r(p)"
table (command) (result), ///
	command (`diff_eq' : ttest share_female, by(everconcent)) ///
	command (`diff_eq' : ttest share_female_lowSE, by(everconcent)) ///
	command (`diff_eq' : ttest avg_score, by(everconcent)) ///
	command (`diff_eq' : ttest female_avg_score, by(everconcent)) ///
	command (`diff_eq' : ttest fem_avg_score_HSES, by(everconcent)) ///
	command (`diff_eq' : ttest fem_avg_score_LSES, by(everconcent)) ///
	command (`diff_eq' : ttest male_avg_score, by(everconcent)) ///
	command (`diff_eq' : ttest fm_ratio, by(everconcent))
	
collect dims
collect label list command, all 
collect label levels command 1 "% female (1)"  ///
							2 "% female from L-SES(2)" ///
                            3 "Total avg. score (3)" ///
                            4 "Female avg. score (4)" ///
							5 "High-SES Female avg. score (5)" ///
							6 "Low-SES Female avg. score (6)" ///
							7 "Male avg. score (7)" ///
                            8 "Female to male avg. score ratio (8)" ///
                             , replace
					
collect style cell result[Non_CEELM CEELM Difference], nformat(%8.3f)
collect style cell result[pvalue], nformat(%8.3f)
collect preview

putdocx clear
putdocx begin
putdocx paragraph, style(Title) font(,16)
putdocx text ("Pre-quota difference - T_test table for outcomes in pre-2016 between control & treated units")
putdocx collect
putdocx save "${tables}/appendix/TabelB3_1_pre2016_difference.docx", replace
restore 


******** 	-------------------------------------	**********
*	Step 3.6: Tab_B3.2: Pre-2016 differences by CEELM status
******** 	-------------------------------------	**********

preserve

keep if year_kankor<2016
local diff_eq "Non_CEELM = r(mu_1) CEELM = r(mu_2) Difference = (r(mu_2)-r(mu_1)) pvalue = r(p)"
table (command) (result), ///
	command (`diff_eq' : ttest share_female, by(ceelm)) ///
	command (`diff_eq' : ttest share_female_lowSE, by(ceelm)) ///
	command (`diff_eq' : ttest avg_score, by(ceelm)) ///
	command (`diff_eq' : ttest female_avg_score, by(ceelm)) ///
	command (`diff_eq' : ttest fem_avg_score_HSES, by(ceelm)) ///
	command (`diff_eq' : ttest fem_avg_score_LSES, by(ceelm)) ///
	command (`diff_eq' : ttest male_avg_score, by(ceelm)) ///
	command (`diff_eq' : ttest fm_ratio, by(ceelm))
	
collect dims
collect label list command, all 
collect label levels command 1 "% female (1)"  ///
							2 "% female from L-SES(2)" ///
                            3 "Total avg. score (3)" ///
                            4 "Female avg. score (4)" ///
							5 "High-SES Female avg. score (5)" ///
							6 "Low-SES Female avg. score (6)" ///
							7 "Male avg. score (7)" ///
                            8 "Female to male avg. score ratio (8)" ///
                             , replace
					
collect style cell result[Non_CEELM CEELM Difference], nformat(%8.3f)
collect style cell result[pvalue], nformat(%8.3f)
collect preview

putdocx clear
putdocx begin
putdocx paragraph, style(Title) font(,16)
putdocx text ("Heterogeneous Module - T_test table for outcomes in pre-2016 between CEELM & Non_CEELM")
putdocx collect
putdocx save "${tables}/appendix/TabelB3_2_pre2016_CEELM.docx", replace
restore


******** 	-------------------------------------	**********
*	Step 3.7: Tabel B5.1 - Major results
******** 	-------------------------------------	**********

use "${data}/uni_by_major.dta", clear // load the datset 

sort major, stable

lab var event_maj "cohort identifier"

gen uni_year=uni*year_kankor
gen maj_year=acdem_program*year_kankor

				* ------------- * * ------------- *  
				* 	Set local	  *
				* ------------- * * ------------- * 
				
local dep_vars_major "share_female share_female_lowSE avg_score female_avg_score fem_avg_score_HSES fem_avg_score_LSES male_avg_score fm_ratio"

* ------------- *
** #1 OLS estimator
  
  foreach var of local dep_vars_major {
eststo : quietly reghdfe `var' i.treat, a(i.major i.uni##i.year_kankor i.acdem_program##i.year_kankor i.year_kankor) cl(major)
    sum `var' if e(sample) & evermajor==0
    scalar evnt1 = r(mean)
	estadd scalar m_`var' = evnt1
}

gen stu_samp=1 if _est_est1==1

esttab using "${tables}/appendix/TabB5_1_OLS.tex", replace ///
cells(b(star fmt(2)) se(par fmt(2)))  starlevels(* 0.10 ** 0.05 *** 0.01) stats(r2 N m_share_female m_share_female_lowSE m_avg_score m_female_avg_score m_fem_avg_score_HSES m_fem_avg_score_LSES m_male_avg_score m_fm_ratio,  fmt(2 2 2 2 2 2 2 2 2 2) labels("R-squared" "N" "Mean % fem" "%fem-lses" "avg_sc" "fem_scr" "fem_hses_scor" "fem_lses_scor" "male_scor" "fm_ratio")) keep(1.treat) varlabels(1.treat "treat") collabels(none) mlabels("% fem" "%fem-lses" "avg_sc" "fem_scr" "fem_hses_scor" "fem_lses_scor" "male_scor" "fm_ratio")  title("Table B5.1- OLS Effects of gender quota on outcomes at MAJOR level")  modelwidth(11) addnote("")

  estimates clear

  
* ------------- *
** #2 Borusyak et al. (2021): Estimation with did_imputation of

foreach var of local dep_vars_major {
eststo : quietly did_imputation `var' major year_kankor event_maj if stu_samp==1, autos fe(uni uni_year acdem_program maj_year)
    sum `var' if e(sample) & evermajor==0
    scalar evnt1 = r(mean)
	estadd scalar m_`var' = evnt1
}

esttab using "${tables}/appendix/TabB5_1_Borusyak.tex", replace ///
cells(b(star fmt(2)) se(par fmt(2))) starlevels(* 0.10 ** 0.05 *** 0.01) stats(r2 N m_share_female m_share_female_lowSE m_avg_score m_female_avg_score m_fem_avg_score_HSES m_fem_avg_score_LSES m_male_avg_score m_fm_ratio,  fmt(2 2 2 2 2 2 2 2 2 2) labels("R-squared" "N" "Mean % fem" "%fem-lses" "avg_sc" "fem_scr" "fem_hses_scor" "fem_lses_scor" "male_scor" "fm_ratio")) keep(tau) varlabels(tau "treat") collabels(none) mlabels("% fem" "%fem-lses" "avg_sc" "fem_scr" "fem_hses_scor" "fem_lses_scor" "male_scor" "fm_ratio")  title("Table #. Effects of gender admission quota - MAJOR - Borusyak ")  modelwidth(11) addnote("")

  estimates clear
  

* ------------- *
** #3 Sun & Abraham Estimator

preserve
sum event_maj
gen nevertreated = evermajor==0 // dummy for the never-treated cohort
forvalues l = 0/2 {
	gen L`l'event = relative_maj==`l'
}
forvalues l = 1/3 {
	gen F`l'event = relative_maj==-`l'
}
drop F1event // normalize K=-1 to zero

// Define a new dataset to store the results
tempname results_data
postfile `results_data' depvar estimate se p_value using results_data.dta, replace

local i = 1
foreach var of local dep_vars_major {
   quietly eventstudyinteract `var' L*event F*event, vce(cluster major) absorb(i.major i.year_kankor i.uni##i.year_kankor i.acdem_program##i.year_kankor) cohort(event_maj) control_cohort(nevertreated)

    matrix b = e(b_iw)
    matrix V = e(V_iw)
    ereturn post b V
    lincom (L0event + L1event + L2event)/3

    scalar b_ht = r(estimate)
    scalar se_ht = r(se)
	scalar p_value =r(p)

    // Store the estimates and standard errors in the dataset
    post `results_data' (`i') (b_ht) (se_ht) (p_value)

    local ++i
}

// Close the postfile
postclose `results_data'

// Load the dataset and display the table
use results_data.dta, clear
list

// Export the dataset to a CSV file
outsheet using "${tables}/appendix/TabB5_1_SunAbraham.csv", comma replace
  
  estimates clear
  
restore

set graphics on

