version 17 
clear all 

********************************************************************************
* PROJECT: Job Market Paper
* TITLE: Closing the gap: Effect of a gender quota on women's access to education in Afghanistan
*******************************************************************************

*** Outline:
	*	Part 0: 	  Set initial configuration and globals 
	*	Part 1: 	  Data preparation 
	*	Part 2: 	  Analysis
	*	Part 3: 	  Appendix

*** OBSERVATIONS:	

*** ID VAR (concentration):		concent

*** WRITTEN BY:		Rafiuddin Najam

*** LAST TIME MODIFIED:		10/9/2023

********************************************************************************
* 					PART 0: Set initial configurations and globals
*******************************************************************************

*do file for cleaning and labeling data if working with excel
global ProjQuota "C:/Temp/RR_AFG_2023_33-v01/Reproducibility package" // personal computer


* global ProjQuota "C:\Users\wb615333\Dropbox\Research_Work\Quota_2016_JMP\" // wb computer


**** Setting up folders
	global data			"${ProjQuota}/data"
	global dofiles		"${ProjQuota}/dofiles"
	global outputs		"${ProjQuota}/outputs"
	global tables		"${outputs}/tables"
	global figures		"${outputs}/figures"

	global main_tables        1   /// See switch
	
	sysdir set PLUS "${dofiles}/ado"

	
*** Install required packages

local packages estout ietoolkit reghdfe did_imputation ///
eventstudyinteract ftools event_plot avar coefplot boottest

foreach package in `packages' {
	capture which `package'
	if _rc == 111 { 
		ssc install `package'
	}
}

	ieboilstart, version(17.0)	
	`r(version)'

********************************************************************************
*	Part 1:  Data Preparation
********************************************************************************
* This code is commented out as it can't be run without the kankor data, which is not shared in the present package
*** Clean and aggregate individual data to concentration and major level
	*** Execute dofiles
	*if (${main_tables} == 1) * {
	*	do "${dofiles}/01_PrepData.do"
	* }

********************************************************************************
*	Part 2:  Data analysis
********************************************************************************

*** Generate main descriptive, EventStudy and regression results
	*** Execute dofiles
	
	if (${main_tables} == 1) {
		do "${dofiles}/02.1_Analysis_Descriptive.do" 
		do "${dofiles}/02.2_Analysis_EventStudy.do"
		do "${dofiles}/02.3_Analysis_Regression.do"
	}


********************************************************************************
*	Part 3: Appendix files 
********************************************************************************

*** Produce additional analytical files here
	*** Execute dofiles
	
	if (${main_tables} == 1) {
		do "${dofiles}/03_Appendix.do" 
	}
